# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/pool-incentives/v1beta1/gov.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from osmosis.pool_incentives.v1beta1 import incentives_pb2 as osmosis_dot_pool__incentives_dot_v1beta1_dot_incentives__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)osmosis/pool-incentives/v1beta1/gov.proto\x12\x1eosmosis.poolincentives.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x30osmosis/pool-incentives/v1beta1/incentives.proto\"\x95\x01\n\x1dReplacePoolIncentivesProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x42\n\x07records\x18\x03 \x03(\x0b\x32+.osmosis.poolincentives.v1beta1.DistrRecordB\x04\xc8\xde\x1f\x00:\x0c\xe8\xa0\x1f\x01\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"\x94\x01\n\x1cUpdatePoolIncentivesProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x42\n\x07records\x18\x03 \x03(\x0b\x32+.osmosis.poolincentives.v1beta1.DistrRecordB\x04\xc8\xde\x1f\x00:\x0c\xe8\xa0\x1f\x01\x88\xa0\x1f\x00\x98\xa0\x1f\x00\x42=Z;github.com/osmosis-labs/osmosis/v14/x/pool-incentives/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.pool_incentives.v1beta1.gov_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z;github.com/osmosis-labs/osmosis/v14/x/pool-incentives/types'
  _REPLACEPOOLINCENTIVESPROPOSAL.fields_by_name['records']._options = None
  _REPLACEPOOLINCENTIVESPROPOSAL.fields_by_name['records']._serialized_options = b'\310\336\037\000'
  _REPLACEPOOLINCENTIVESPROPOSAL._options = None
  _REPLACEPOOLINCENTIVESPROPOSAL._serialized_options = b'\350\240\037\001\210\240\037\000\230\240\037\000'
  _UPDATEPOOLINCENTIVESPROPOSAL.fields_by_name['records']._options = None
  _UPDATEPOOLINCENTIVESPROPOSAL.fields_by_name['records']._serialized_options = b'\310\336\037\000'
  _UPDATEPOOLINCENTIVESPROPOSAL._options = None
  _UPDATEPOOLINCENTIVESPROPOSAL._serialized_options = b'\350\240\037\001\210\240\037\000\230\240\037\000'
  _REPLACEPOOLINCENTIVESPROPOSAL._serialized_start=150
  _REPLACEPOOLINCENTIVESPROPOSAL._serialized_end=299
  _UPDATEPOOLINCENTIVESPROPOSAL._serialized_start=302
  _UPDATEPOOLINCENTIVESPROPOSAL._serialized_end=450
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import osmosis.txfees.v1beta1.query_pb2

class QueryStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    FeeTokens: grpc.UnaryUnaryMultiCallable[
        osmosis.txfees.v1beta1.query_pb2.QueryFeeTokensRequest,
        osmosis.txfees.v1beta1.query_pb2.QueryFeeTokensResponse,
    ]
    """FeeTokens returns a list of all the whitelisted fee tokens and their
    corresponding pools. It does not include the BaseDenom, which has its own
    query endpoint
    """
    DenomSpotPrice: grpc.UnaryUnaryMultiCallable[
        osmosis.txfees.v1beta1.query_pb2.QueryDenomSpotPriceRequest,
        osmosis.txfees.v1beta1.query_pb2.QueryDenomSpotPriceResponse,
    ]
    """DenomSpotPrice returns all spot prices by each registered token denom."""
    DenomPoolId: grpc.UnaryUnaryMultiCallable[
        osmosis.txfees.v1beta1.query_pb2.QueryDenomPoolIdRequest,
        osmosis.txfees.v1beta1.query_pb2.QueryDenomPoolIdResponse,
    ]
    """Returns the poolID for a specified denom input."""
    BaseDenom: grpc.UnaryUnaryMultiCallable[
        osmosis.txfees.v1beta1.query_pb2.QueryBaseDenomRequest,
        osmosis.txfees.v1beta1.query_pb2.QueryBaseDenomResponse,
    ]
    """Returns a list of all base denom tokens and their corresponding pools."""

class QueryServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def FeeTokens(
        self,
        request: osmosis.txfees.v1beta1.query_pb2.QueryFeeTokensRequest,
        context: grpc.ServicerContext,
    ) -> osmosis.txfees.v1beta1.query_pb2.QueryFeeTokensResponse:
        """FeeTokens returns a list of all the whitelisted fee tokens and their
        corresponding pools. It does not include the BaseDenom, which has its own
        query endpoint
        """
    @abc.abstractmethod
    def DenomSpotPrice(
        self,
        request: osmosis.txfees.v1beta1.query_pb2.QueryDenomSpotPriceRequest,
        context: grpc.ServicerContext,
    ) -> osmosis.txfees.v1beta1.query_pb2.QueryDenomSpotPriceResponse:
        """DenomSpotPrice returns all spot prices by each registered token denom."""
    @abc.abstractmethod
    def DenomPoolId(
        self,
        request: osmosis.txfees.v1beta1.query_pb2.QueryDenomPoolIdRequest,
        context: grpc.ServicerContext,
    ) -> osmosis.txfees.v1beta1.query_pb2.QueryDenomPoolIdResponse:
        """Returns the poolID for a specified denom input."""
    @abc.abstractmethod
    def BaseDenom(
        self,
        request: osmosis.txfees.v1beta1.query_pb2.QueryBaseDenomRequest,
        context: grpc.ServicerContext,
    ) -> osmosis.txfees.v1beta1.query_pb2.QueryBaseDenomResponse:
        """Returns a list of all base denom tokens and their corresponding pools."""

def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
Implementing SHA-256 from scratch was fun, but for RIPEMD160 I am
taking an existing implementation and made some cleanups and api changes.


ripemd.py - pure Python implementation of the RIPEMD-160 algorithm.
Bjorn Edstrom <be@bjrn.se> 16 december 2007.

Copyrights
==========

This code is a derived from an implementation by Markus Friedl which is
subject to the following license. This Python implementation is not
subject to any other license.

/*
* Copyright (c) 2001 Markus Friedl.  All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE,
* DATA, OR PROFITS OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
* Preneel, Bosselaers, Dobbertin, "The Cryptographic Hash Function RIPEMD-160",
* RSA Laboratories, CryptoBytes, Volume 3, Number 2, Autumn 1997,
* ftp://ftp.rsasecurity.com/pub/cryptobytes/crypto3n2.pdf
*/
"""

import struct
import sys

# -----------------------------------------------------------------------------
# public interface


def ripemd160(b: bytes) -> bytes:
    """simple wrapper for a simpler API to this hash function, just bytes to bytes

    Args:
        b: the bytes input
    """
    ctx = RMDContext()
    RMD160Update(ctx, b, len(b))
    digest = RMD160Final(ctx)
    return digest


# -----------------------------------------------------------------------------


class RMDContext:
    def __init__(self):
        self.state = [
            0x67452301,
            0xEFCDAB89,
            0x98BADCFE,
            0x10325476,
            0xC3D2E1F0,
        ]  # uint32
        self.count = 0  # uint64
        self.buffer = [0] * 64  # uchar


def RMD160Update(ctx, inp, inplen):
    have = int((ctx.count // 8) % 64)
    inplen = int(inplen)
    need = 64 - have
    ctx.count += 8 * inplen
    off = 0
    if inplen >= need:
        if have:
            for i in range(need):
                ctx.buffer[have + i] = inp[i]
            RMD160Transform(ctx.state, ctx.buffer)
            off = need
            have = 0
        while off + 64 <= inplen:
            RMD160Transform(ctx.state, inp[off:])  # <---
            off += 64
    if off < inplen:
        for i in range(inplen - off):
            ctx.buffer[have + i] = inp[off + i]


def RMD160Final(ctx):
    size = struct.pack("<Q", ctx.count)
    padlen = 64 - ((ctx.count // 8) % 64)
    if padlen < 1 + 8:
        padlen += 64
    RMD160Update(ctx, PADDING, padlen - 8)
    RMD160Update(ctx, size, 8)
    return struct.pack("<5L", *ctx.state)


# -----------------------------------------------------------------------------

K0 = 0x00000000
K1 = 0x5A827999
K2 = 0x6ED9EBA1
K3 = 0x8F1BBCDC
K4 = 0xA953FD4E
KK0 = 0x50A28BE6
KK1 = 0x5C4DD124
KK2 = 0x6D703EF3
KK3 = 0x7A6D76E9
KK4 = 0x00000000

PADDING = [0x80] + [0] * 63


def ROL(n, x):
    return ((x << n) & 0xFFFFFFFF) | (x >> (32 - n))


def F0(x, y, z):
    return x ^ y ^ z


def F1(x, y, z):
    return (x & y) | (((~x) % 0x100000000) & z)


def F2(x, y, z):
    return (x | ((~y) % 0x100000000)) ^ z


def F3(x, y, z):
    return (x & z) | (((~z) % 0x100000000) & y)


def F4(x, y, z):
    return x ^ (y | ((~z) % 0x100000000))


def R(a, b, c, d, e, Fj, Kj, sj, rj, X):
    a = ROL(sj, (a + Fj(b, c, d) + X[rj] + Kj) % 0x100000000) + e
    c = ROL(10, c)
    return a % 0x100000000, c


def RMD160Transform(state, block):  # uint32 state[5], uchar block[64]
    x = [0] * 16
    assert (
        sys.byteorder == 'little'
    ), "Only little endian is supported atm for RIPEMD160"
    x = struct.unpack('<16L', bytes(block[0:64]))

    a = state[0]
    b = state[1]
    c = state[2]
    d = state[3]
    e = state[4]

    # /* Round 1 */
    a, c = R(a, b, c, d, e, F0, K0, 11, 0, x)
    e, b = R(e, a, b, c, d, F0, K0, 14, 1, x)
    d, a = R(d, e, a, b, c, F0, K0, 15, 2, x)
    c, e = R(c, d, e, a, b, F0, K0, 12, 3, x)
    b, d = R(b, c, d, e, a, F0, K0, 5, 4, x)
    a, c = R(a, b, c, d, e, F0, K0, 8, 5, x)
    e, b = R(e, a, b, c, d, F0, K0, 7, 6, x)
    d, a = R(d, e, a, b, c, F0, K0, 9, 7, x)
    c, e = R(c, d, e, a, b, F0, K0, 11, 8, x)
    b, d = R(b, c, d, e, a, F0, K0, 13, 9, x)
    a, c = R(a, b, c, d, e, F0, K0, 14, 10, x)
    e, b = R(e, a, b, c, d, F0, K0, 15, 11, x)
    d, a = R(d, e, a, b, c, F0, K0, 6, 12, x)
    c, e = R(c, d, e, a, b, F0, K0, 7, 13, x)
    b, d = R(b, c, d, e, a, F0, K0, 9, 14, x)
    a, c = R(a, b, c, d, e, F0, K0, 8, 15, x)  # /* #15 */
    # /* Round 2 */
    e, b = R(e, a, b, c, d, F1, K1, 7, 7, x)
    d, a = R(d, e, a, b, c, F1, K1, 6, 4, x)
    c, e = R(c, d, e, a, b, F1, K1, 8, 13, x)
    b, d = R(b, c, d, e, a, F1, K1, 13, 1, x)
    a, c = R(a, b, c, d, e, F1, K1, 11, 10, x)
    e, b = R(e, a, b, c, d, F1, K1, 9, 6, x)
    d, a = R(d, e, a, b, c, F1, K1, 7, 15, x)
    c, e = R(c, d, e, a, b, F1, K1, 15, 3, x)
    b, d = R(b, c, d, e, a, F1, K1, 7, 12, x)
    a, c = R(a, b, c, d, e, F1, K1, 12, 0, x)
    e, b = R(e, a, b, c, d, F1, K1, 15, 9, x)
    d, a = R(d, e, a, b, c, F1, K1, 9, 5, x)
    c, e = R(c, d, e, a, b, F1, K1, 11, 2, x)
    b, d = R(b, c, d, e, a, F1, K1, 7, 14, x)
    a, c = R(a, b, c, d, e, F1, K1, 13, 11, x)
    e, b = R(e, a, b, c, d, F1, K1, 12, 8, x)  # /* #31 */
    # /* Round 3 */
    d, a = R(d, e, a, b, c, F2, K2, 11, 3, x)
    c, e = R(c, d, e, a, b, F2, K2, 13, 10, x)
    b, d = R(b, c, d, e, a, F2, K2, 6, 14, x)
    a, c = R(a, b, c, d, e, F2, K2, 7, 4, x)
    e, b = R(e, a, b, c, d, F2, K2, 14, 9, x)
    d, a = R(d, e, a, b, c, F2, K2, 9, 15, x)
    c, e = R(c, d, e, a, b, F2, K2, 13, 8, x)
    b, d = R(b, c, d, e, a, F2, K2, 15, 1, x)
    a, c = R(a, b, c, d, e, F2, K2, 14, 2, x)
    e, b = R(e, a, b, c, d, F2, K2, 8, 7, x)
    d, a = R(d, e, a, b, c, F2, K2, 13, 0, x)
    c, e = R(c, d, e, a, b, F2, K2, 6, 6, x)
    b, d = R(b, c, d, e, a, F2, K2, 5, 13, x)
    a, c = R(a, b, c, d, e, F2, K2, 12, 11, x)
    e, b = R(e, a, b, c, d, F2, K2, 7, 5, x)
    d, a = R(d, e, a, b, c, F2, K2, 5, 12, x)  # /* #47 */
    # /* Round 4 */
    c, e = R(c, d, e, a, b, F3, K3, 11, 1, x)
    b, d = R(b, c, d, e, a, F3, K3, 12, 9, x)
    a, c = R(a, b, c, d, e, F3, K3, 14, 11, x)
    e, b = R(e, a, b, c, d, F3, K3, 15, 10, x)
    d, a = R(d, e, a, b, c, F3, K3, 14, 0, x)
    c, e = R(c, d, e, a, b, F3, K3, 15, 8, x)
    b, d = R(b, c, d, e, a, F3, K3, 9, 12, x)
    a, c = R(a, b, c, d, e, F3, K3, 8, 4, x)
    e, b = R(e, a, b, c, d, F3, K3, 9, 13, x)
    d, a = R(d, e, a, b, c, F3, K3, 14, 3, x)
    c, e = R(c, d, e, a, b, F3, K3, 5, 7, x)
    b, d = R(b, c, d, e, a, F3, K3, 6, 15, x)
    a, c = R(a, b, c, d, e, F3, K3, 8, 14, x)
    e, b = R(e, a, b, c, d, F3, K3, 6, 5, x)
    d, a = R(d, e, a, b, c, F3, K3, 5, 6, x)
    c, e = R(c, d, e, a, b, F3, K3, 12, 2, x)  # /* #63 */
    # /* Round 5 */
    b, d = R(b, c, d, e, a, F4, K4, 9, 4, x)
    a, c = R(a, b, c, d, e, F4, K4, 15, 0, x)
    e, b = R(e, a, b, c, d, F4, K4, 5, 5, x)
    d, a = R(d, e, a, b, c, F4, K4, 11, 9, x)
    c, e = R(c, d, e, a, b, F4, K4, 6, 7, x)
    b, d = R(b, c, d, e, a, F4, K4, 8, 12, x)
    a, c = R(a, b, c, d, e, F4, K4, 13, 2, x)
    e, b = R(e, a, b, c, d, F4, K4, 12, 10, x)
    d, a = R(d, e, a, b, c, F4, K4, 5, 14, x)
    c, e = R(c, d, e, a, b, F4, K4, 12, 1, x)
    b, d = R(b, c, d, e, a, F4, K4, 13, 3, x)
    a, c = R(a, b, c, d, e, F4, K4, 14, 8, x)
    e, b = R(e, a, b, c, d, F4, K4, 11, 11, x)
    d, a = R(d, e, a, b, c, F4, K4, 8, 6, x)
    c, e = R(c, d, e, a, b, F4, K4, 5, 15, x)
    b, d = R(b, c, d, e, a, F4, K4, 6, 13, x)  # /* #79 */

    aa = a
    bb = b
    cc = c
    dd = d
    ee = e

    a = state[0]
    b = state[1]
    c = state[2]
    d = state[3]
    e = state[4]

    # /* Parallel round 1 */
    a, c = R(a, b, c, d, e, F4, KK0, 8, 5, x)
    e, b = R(e, a, b, c, d, F4, KK0, 9, 14, x)
    d, a = R(d, e, a, b, c, F4, KK0, 9, 7, x)
    c, e = R(c, d, e, a, b, F4, KK0, 11, 0, x)
    b, d = R(b, c, d, e, a, F4, KK0, 13, 9, x)
    a, c = R(a, b, c, d, e, F4, KK0, 15, 2, x)
    e, b = R(e, a, b, c, d, F4, KK0, 15, 11, x)
    d, a = R(d, e, a, b, c, F4, KK0, 5, 4, x)
    c, e = R(c, d, e, a, b, F4, KK0, 7, 13, x)
    b, d = R(b, c, d, e, a, F4, KK0, 7, 6, x)
    a, c = R(a, b, c, d, e, F4, KK0, 8, 15, x)
    e, b = R(e, a, b, c, d, F4, KK0, 11, 8, x)
    d, a = R(d, e, a, b, c, F4, KK0, 14, 1, x)
    c, e = R(c, d, e, a, b, F4, KK0, 14, 10, x)
    b, d = R(b, c, d, e, a, F4, KK0, 12, 3, x)
    a, c = R(a, b, c, d, e, F4, KK0, 6, 12, x)  # /* #15 */
    # /* Parallel round 2 */
    e, b = R(e, a, b, c, d, F3, KK1, 9, 6, x)
    d, a = R(d, e, a, b, c, F3, KK1, 13, 11, x)
    c, e = R(c, d, e, a, b, F3, KK1, 15, 3, x)
    b, d = R(b, c, d, e, a, F3, KK1, 7, 7, x)
    a, c = R(a, b, c, d, e, F3, KK1, 12, 0, x)
    e, b = R(e, a, b, c, d, F3, KK1, 8, 13, x)
    d, a = R(d, e, a, b, c, F3, KK1, 9, 5, x)
    c, e = R(c, d, e, a, b, F3, KK1, 11, 10, x)
    b, d = R(b, c, d, e, a, F3, KK1, 7, 14, x)
    a, c = R(a, b, c, d, e, F3, KK1, 7, 15, x)
    e, b = R(e, a, b, c, d, F3, KK1, 12, 8, x)
    d, a = R(d, e, a, b, c, F3, KK1, 7, 12, x)
    c, e = R(c, d, e, a, b, F3, KK1, 6, 4, x)
    b, d = R(b, c, d, e, a, F3, KK1, 15, 9, x)
    a, c = R(a, b, c, d, e, F3, KK1, 13, 1, x)
    e, b = R(e, a, b, c, d, F3, KK1, 11, 2, x)  # /* #31 */
    # /* Parallel round 3 */
    d, a = R(d, e, a, b, c, F2, KK2, 9, 15, x)
    c, e = R(c, d, e, a, b, F2, KK2, 7, 5, x)
    b, d = R(b, c, d, e, a, F2, KK2, 15, 1, x)
    a, c = R(a, b, c, d, e, F2, KK2, 11, 3, x)
    e, b = R(e, a, b, c, d, F2, KK2, 8, 7, x)
    d, a = R(d, e, a, b, c, F2, KK2, 6, 14, x)
    c, e = R(c, d, e, a, b, F2, KK2, 6, 6, x)
    b, d = R(b, c, d, e, a, F2, KK2, 14, 9, x)
    a, c = R(a, b, c, d, e, F2, KK2, 12, 11, x)
    e, b = R(e, a, b, c, d, F2, KK2, 13, 8, x)
    d, a = R(d, e, a, b, c, F2, KK2, 5, 12, x)
    c, e = R(c, d, e, a, b, F2, KK2, 14, 2, x)
    b, d = R(b, c, d, e, a, F2, KK2, 13, 10, x)
    a, c = R(a, b, c, d, e, F2, KK2, 13, 0, x)
    e, b = R(e, a, b, c, d, F2, KK2, 7, 4, x)
    d, a = R(d, e, a, b, c, F2, KK2, 5, 13, x)  # /* #47 */
    # /* Parallel round 4 */
    c, e = R(c, d, e, a, b, F1, KK3, 15, 8, x)
    b, d = R(b, c, d, e, a, F1, KK3, 5, 6, x)
    a, c = R(a, b, c, d, e, F1, KK3, 8, 4, x)
    e, b = R(e, a, b, c, d, F1, KK3, 11, 1, x)
    d, a = R(d, e, a, b, c, F1, KK3, 14, 3, x)
    c, e = R(c, d, e, a, b, F1, KK3, 14, 11, x)
    b, d = R(b, c, d, e, a, F1, KK3, 6, 15, x)
    a, c = R(a, b, c, d, e, F1, KK3, 14, 0, x)
    e, b = R(e, a, b, c, d, F1, KK3, 6, 5, x)
    d, a = R(d, e, a, b, c, F1, KK3, 9, 12, x)
    c, e = R(c, d, e, a, b, F1, KK3, 12, 2, x)
    b, d = R(b, c, d, e, a, F1, KK3, 9, 13, x)
    a, c = R(a, b, c, d, e, F1, KK3, 12, 9, x)
    e, b = R(e, a, b, c, d, F1, KK3, 5, 7, x)
    d, a = R(d, e, a, b, c, F1, KK3, 15, 10, x)
    c, e = R(c, d, e, a, b, F1, KK3, 8, 14, x)  # /* #63 */
    # /* Parallel round 5 */
    b, d = R(b, c, d, e, a, F0, KK4, 8, 12, x)
    a, c = R(a, b, c, d, e, F0, KK4, 5, 15, x)
    e, b = R(e, a, b, c, d, F0, KK4, 12, 10, x)
    d, a = R(d, e, a, b, c, F0, KK4, 9, 4, x)
    c, e = R(c, d, e, a, b, F0, KK4, 12, 1, x)
    b, d = R(b, c, d, e, a, F0, KK4, 5, 5, x)
    a, c = R(a, b, c, d, e, F0, KK4, 14, 8, x)
    e, b = R(e, a, b, c, d, F0, KK4, 6, 7, x)
    d, a = R(d, e, a, b, c, F0, KK4, 8, 6, x)
    c, e = R(c, d, e, a, b, F0, KK4, 13, 2, x)
    b, d = R(b, c, d, e, a, F0, KK4, 6, 13, x)
    a, c = R(a, b, c, d, e, F0, KK4, 5, 14, x)
    e, b = R(e, a, b, c, d, F0, KK4, 15, 0, x)
    d, a = R(d, e, a, b, c, F0, KK4, 13, 3, x)
    c, e = R(c, d, e, a, b, F0, KK4, 11, 9, x)
    b, d = R(b, c, d, e, a, F0, KK4, 11, 11, x)  # /* #79 */

    t = (state[1] + cc + d) % 0x100000000
    state[1] = (state[2] + dd + e) % 0x100000000
    state[2] = (state[3] + ee + a) % 0x100000000
    state[3] = (state[4] + aa + b) % 0x100000000
    state[4] = (state[0] + bb + c) % 0x100000000
    state[0] = t % 0x100000000



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
this is a legacy package that requires additional migration logic
in order to use the correct packge. Decision made to use legacy package path
until clear steps for migration logic and the unknowns for state breaking are
investigated for changing proto package.
"""
import builtins
import collections.abc
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class SmoothWeightChangeParams(google.protobuf.message.Message):
    """Parameters for changing the weights in a balancer pool smoothly from
    a start weight and end weight over a period of time.
    Currently, the only smooth change supported is linear changing between
    the two weights, but more types may be added in the future.
    When these parameters are set, the weight w(t) for pool time `t` is the
    following:
      t <= start_time: w(t) = initial_pool_weights
      start_time < t <= start_time + duration:
        w(t) = initial_pool_weights + (t - start_time) *
          (target_pool_weights - initial_pool_weights) / (duration)
      t > start_time + duration: w(t) = target_pool_weights
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    START_TIME_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    INITIAL_POOL_WEIGHTS_FIELD_NUMBER: builtins.int
    TARGET_POOL_WEIGHTS_FIELD_NUMBER: builtins.int
    @property
    def start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """The start time for beginning the weight change.
        If a parameter change / pool instantiation leaves this blank,
        it should be generated by the state_machine as the current time.
        """
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """Duration for the weights to change over"""
    @property
    def initial_pool_weights(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PoolAsset]:
        """The initial pool weights. These are copied from the pool's settings
        at the time of weight change instantiation.
        The amount PoolAsset.token.amount field is ignored if present,
        future type refactorings should just have a type with the denom & weight
        here.
        """
    @property
    def target_pool_weights(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PoolAsset]:
        """The target pool weights. The pool weights will change linearly with respect
        to time between start_time, and start_time + duration. The amount
        PoolAsset.token.amount field is ignored if present, future type
        refactorings should just have a type with the denom & weight here.
        Intermediate variable for the 'slope' of pool weights. This is equal to
        (target_pool_weights - initial_pool_weights) / (duration)
        TODO: Work out precision, and decide if this is good to add
        repeated PoolAsset poolWeightSlope = 5 [
         (gogoproto.moretags) = "yaml:\\"pool_weight_slope\\"",
         (gogoproto.nullable) = false
        ];
        """
    def __init__(
        self,
        *,
        start_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
        initial_pool_weights: collections.abc.Iterable[global___PoolAsset] | None = ...,
        target_pool_weights: collections.abc.Iterable[global___PoolAsset] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration", "start_time", b"start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["duration", b"duration", "initial_pool_weights", b"initial_pool_weights", "start_time", b"start_time", "target_pool_weights", b"target_pool_weights"]) -> None: ...

global___SmoothWeightChangeParams = SmoothWeightChangeParams

@typing_extensions.final
class PoolParams(google.protobuf.message.Message):
    """PoolParams defined the parameters that will be managed by the pool
    governance in the future. This params are not managed by the chain
    governance. Instead they will be managed by the token holders of the pool.
    The pool's token holders are specified in future_pool_governor.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SWAP_FEE_FIELD_NUMBER: builtins.int
    EXIT_FEE_FIELD_NUMBER: builtins.int
    SMOOTH_WEIGHT_CHANGE_PARAMS_FIELD_NUMBER: builtins.int
    swap_fee: builtins.str
    exit_fee: builtins.str
    @property
    def smooth_weight_change_params(self) -> global___SmoothWeightChangeParams: ...
    def __init__(
        self,
        *,
        swap_fee: builtins.str = ...,
        exit_fee: builtins.str = ...,
        smooth_weight_change_params: global___SmoothWeightChangeParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["smooth_weight_change_params", b"smooth_weight_change_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["exit_fee", b"exit_fee", "smooth_weight_change_params", b"smooth_weight_change_params", "swap_fee", b"swap_fee"]) -> None: ...

global___PoolParams = PoolParams

@typing_extensions.final
class PoolAsset(google.protobuf.message.Message):
    """Pool asset is an internal struct that combines the amount of the
    token in the pool, and its balancer weight.
    This is an awkward packaging of data,
    and should be revisited in a future state migration.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOKEN_FIELD_NUMBER: builtins.int
    WEIGHT_FIELD_NUMBER: builtins.int
    @property
    def token(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Coins we are talking about,
        the denomination must be unique amongst all PoolAssets for this pool.
        """
    weight: builtins.str
    """Weight that is not normalized. This weight must be less than 2^50"""
    def __init__(
        self,
        *,
        token: cosmos.base.v1beta1.coin_pb2.Coin | None = ...,
        weight: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token", b"token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token", b"token", "weight", b"weight"]) -> None: ...

global___PoolAsset = PoolAsset

@typing_extensions.final
class Pool(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    POOL_PARAMS_FIELD_NUMBER: builtins.int
    FUTURE_POOL_GOVERNOR_FIELD_NUMBER: builtins.int
    TOTAL_SHARES_FIELD_NUMBER: builtins.int
    POOL_ASSETS_FIELD_NUMBER: builtins.int
    TOTAL_WEIGHT_FIELD_NUMBER: builtins.int
    address: builtins.str
    id: builtins.int
    @property
    def pool_params(self) -> global___PoolParams: ...
    future_pool_governor: builtins.str
    """This string specifies who will govern the pool in the future.
    Valid forms of this are:
    {token name},{duration}
    {duration}
    where {token name} if specified is the token which determines the
    governor, and if not specified is the LP token for this pool.duration is
    a time specified as 0w,1w,2w, etc. which specifies how long the token
    would need to be locked up to count in governance. 0w means no lockup.
    TODO: Further improve these docs
    """
    @property
    def total_shares(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """sum of all LP tokens sent out"""
    @property
    def pool_assets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PoolAsset]:
        """These are assumed to be sorted by denomiation.
        They contain the pool asset and the information about the weight
        """
    total_weight: builtins.str
    """sum of all non-normalized pool weights"""
    def __init__(
        self,
        *,
        address: builtins.str = ...,
        id: builtins.int = ...,
        pool_params: global___PoolParams | None = ...,
        future_pool_governor: builtins.str = ...,
        total_shares: cosmos.base.v1beta1.coin_pb2.Coin | None = ...,
        pool_assets: collections.abc.Iterable[global___PoolAsset] | None = ...,
        total_weight: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_params", b"pool_params", "total_shares", b"total_shares"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "future_pool_governor", b"future_pool_governor", "id", b"id", "pool_assets", b"pool_assets", "pool_params", b"pool_params", "total_shares", b"total_shares", "total_weight", b"total_weight"]) -> None: ...

global___Pool = Pool



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Params(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MINTED_DENOM_FIELD_NUMBER: builtins.int
    minted_denom: builtins.str
    """minted_denom is the denomination of the coin expected to be minted by the
    minting module. Pool-incentives module doesn’t actually mint the coin
    itself, but rather manages the distribution of coins that matches the
    defined minted_denom.
    """
    def __init__(
        self,
        *,
        minted_denom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["minted_denom", b"minted_denom"]) -> None: ...

global___Params = Params

@typing_extensions.final
class LockableDurationsInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOCKABLE_DURATIONS_FIELD_NUMBER: builtins.int
    @property
    def lockable_durations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.duration_pb2.Duration]: ...
    def __init__(
        self,
        *,
        lockable_durations: collections.abc.Iterable[google.protobuf.duration_pb2.Duration] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lockable_durations", b"lockable_durations"]) -> None: ...

global___LockableDurationsInfo = LockableDurationsInfo

@typing_extensions.final
class DistrInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOTAL_WEIGHT_FIELD_NUMBER: builtins.int
    RECORDS_FIELD_NUMBER: builtins.int
    total_weight: builtins.str
    @property
    def records(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DistrRecord]: ...
    def __init__(
        self,
        *,
        total_weight: builtins.str = ...,
        records: collections.abc.Iterable[global___DistrRecord] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["records", b"records", "total_weight", b"total_weight"]) -> None: ...

global___DistrInfo = DistrInfo

@typing_extensions.final
class DistrRecord(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GAUGE_ID_FIELD_NUMBER: builtins.int
    WEIGHT_FIELD_NUMBER: builtins.int
    gauge_id: builtins.int
    weight: builtins.str
    def __init__(
        self,
        *,
        gauge_id: builtins.int = ...,
        weight: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["gauge_id", b"gauge_id", "weight", b"weight"]) -> None: ...

global___DistrRecord = DistrRecord

@typing_extensions.final
class PoolToGauge(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POOL_ID_FIELD_NUMBER: builtins.int
    GAUGE_ID_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    gauge_id: builtins.int
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration: ...
    def __init__(
        self,
        *,
        pool_id: builtins.int = ...,
        gauge_id: builtins.int = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["duration", b"duration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["duration", b"duration", "gauge_id", b"gauge_id", "pool_id", b"pool_id"]) -> None: ...

global___PoolToGauge = PoolToGauge

@typing_extensions.final
class PoolToGauges(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POOL_TO_GAUGE_FIELD_NUMBER: builtins.int
    @property
    def pool_to_gauge(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PoolToGauge]: ...
    def __init__(
        self,
        *,
        pool_to_gauge: collections.abc.Iterable[global___PoolToGauge] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_to_gauge", b"pool_to_gauge"]) -> None: ...

global___PoolToGauges = PoolToGauges



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import cosmos.base.v1beta1.coin_pb2
import cosmos.crypto.multisig.v1beta1.multisig_pb2
import cosmos.tx.signing.v1beta1.signing_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Tx(google.protobuf.message.Message):
    """Tx is the standard type used for broadcasting transactions."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BODY_FIELD_NUMBER: builtins.int
    AUTH_INFO_FIELD_NUMBER: builtins.int
    SIGNATURES_FIELD_NUMBER: builtins.int
    @property
    def body(self) -> global___TxBody:
        """body is the processable content of the transaction"""
    @property
    def auth_info(self) -> global___AuthInfo:
        """auth_info is the authorization related content of the transaction,
        specifically signers, signer modes and fee
        """
    @property
    def signatures(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
        """signatures is a list of signatures that matches the length and order of
        AuthInfo's signer_infos to allow connecting signature meta information like
        public key and signing mode by position.
        """
    def __init__(
        self,
        *,
        body: global___TxBody | None = ...,
        auth_info: global___AuthInfo | None = ...,
        signatures: collections.abc.Iterable[builtins.bytes] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["auth_info", b"auth_info", "body", b"body"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["auth_info", b"auth_info", "body", b"body", "signatures", b"signatures"]) -> None: ...

global___Tx = Tx

@typing_extensions.final
class TxRaw(google.protobuf.message.Message):
    """TxRaw is a variant of Tx that pins the signer's exact binary representation
    of body and auth_info. This is used for signing, broadcasting and
    verification. The binary `serialize(tx: TxRaw)` is stored in Tendermint and
    the hash `sha256(serialize(tx: TxRaw))` becomes the "txhash", commonly used
    as the transaction ID.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BODY_BYTES_FIELD_NUMBER: builtins.int
    AUTH_INFO_BYTES_FIELD_NUMBER: builtins.int
    SIGNATURES_FIELD_NUMBER: builtins.int
    body_bytes: builtins.bytes
    """body_bytes is a protobuf serialization of a TxBody that matches the
    representation in SignDoc.
    """
    auth_info_bytes: builtins.bytes
    """auth_info_bytes is a protobuf serialization of an AuthInfo that matches the
    representation in SignDoc.
    """
    @property
    def signatures(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
        """signatures is a list of signatures that matches the length and order of
        AuthInfo's signer_infos to allow connecting signature meta information like
        public key and signing mode by position.
        """
    def __init__(
        self,
        *,
        body_bytes: builtins.bytes = ...,
        auth_info_bytes: builtins.bytes = ...,
        signatures: collections.abc.Iterable[builtins.bytes] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["auth_info_bytes", b"auth_info_bytes", "body_bytes", b"body_bytes", "signatures", b"signatures"]) -> None: ...

global___TxRaw = TxRaw

@typing_extensions.final
class SignDoc(google.protobuf.message.Message):
    """SignDoc is the type used for generating sign bytes for SIGN_MODE_DIRECT."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BODY_BYTES_FIELD_NUMBER: builtins.int
    AUTH_INFO_BYTES_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    ACCOUNT_NUMBER_FIELD_NUMBER: builtins.int
    body_bytes: builtins.bytes
    """body_bytes is protobuf serialization of a TxBody that matches the
    representation in TxRaw.
    """
    auth_info_bytes: builtins.bytes
    """auth_info_bytes is a protobuf serialization of an AuthInfo that matches the
    representation in TxRaw.
    """
    chain_id: builtins.str
    """chain_id is the unique identifier of the chain this transaction targets.
    It prevents signed transactions from being used on another chain by an
    attacker
    """
    account_number: builtins.int
    """account_number is the account number of the account in state"""
    def __init__(
        self,
        *,
        body_bytes: builtins.bytes = ...,
        auth_info_bytes: builtins.bytes = ...,
        chain_id: builtins.str = ...,
        account_number: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_number", b"account_number", "auth_info_bytes", b"auth_info_bytes", "body_bytes", b"body_bytes", "chain_id", b"chain_id"]) -> None: ...

global___SignDoc = SignDoc

@typing_extensions.final
class TxBody(google.protobuf.message.Message):
    """TxBody is the body of a transaction that all signers sign over."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGES_FIELD_NUMBER: builtins.int
    MEMO_FIELD_NUMBER: builtins.int
    TIMEOUT_HEIGHT_FIELD_NUMBER: builtins.int
    EXTENSION_OPTIONS_FIELD_NUMBER: builtins.int
    NON_CRITICAL_EXTENSION_OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def messages(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """messages is a list of messages to be executed. The required signers of
        those messages define the number and order of elements in AuthInfo's
        signer_infos and Tx's signatures. Each required signer address is added to
        the list only the first time it occurs.
        By convention, the first required signer (usually from the first message)
        is referred to as the primary signer and pays the fee for the whole
        transaction.
        """
    memo: builtins.str
    """memo is any arbitrary note/comment to be added to the transaction.
    WARNING: in clients, any publicly exposed text should not be called memo,
    but should be called `note` instead (see https://github.com/cosmos/cosmos-sdk/issues/9122).
    """
    timeout_height: builtins.int
    """timeout is the block height after which this transaction will not
    be processed by the chain
    """
    @property
    def extension_options(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """extension_options are arbitrary options that can be added by chains
        when the default options are not sufficient. If any of these are present
        and can't be handled, the transaction will be rejected
        """
    @property
    def non_critical_extension_options(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """extension_options are arbitrary options that can be added by chains
        when the default options are not sufficient. If any of these are present
        and can't be handled, they will be ignored
        """
    def __init__(
        self,
        *,
        messages: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
        memo: builtins.str = ...,
        timeout_height: builtins.int = ...,
        extension_options: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
        non_critical_extension_options: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["extension_options", b"extension_options", "memo", b"memo", "messages", b"messages", "non_critical_extension_options", b"non_critical_extension_options", "timeout_height", b"timeout_height"]) -> None: ...

global___TxBody = TxBody

@typing_extensions.final
class AuthInfo(google.protobuf.message.Message):
    """AuthInfo describes the fee and signer modes that are used to sign a
    transaction.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SIGNER_INFOS_FIELD_NUMBER: builtins.int
    FEE_FIELD_NUMBER: builtins.int
    @property
    def signer_infos(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SignerInfo]:
        """signer_infos defines the signing modes for the required signers. The number
        and order of elements must match the required signers from TxBody's
        messages. The first element is the primary signer and the one which pays
        the fee.
        """
    @property
    def fee(self) -> global___Fee:
        """Fee is the fee and gas limit for the transaction. The first signer is the
        primary signer and the one which pays the fee. The fee can be calculated
        based on the cost of evaluating the body and doing signature verification
        of the signers. This can be estimated via simulation.
        """
    def __init__(
        self,
        *,
        signer_infos: collections.abc.Iterable[global___SignerInfo] | None = ...,
        fee: global___Fee | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["fee", b"fee"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fee", b"fee", "signer_infos", b"signer_infos"]) -> None: ...

global___AuthInfo = AuthInfo

@typing_extensions.final
class SignerInfo(google.protobuf.message.Message):
    """SignerInfo describes the public key and signing mode of a single top-level
    signer.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PUBLIC_KEY_FIELD_NUMBER: builtins.int
    MODE_INFO_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    @property
    def public_key(self) -> google.protobuf.any_pb2.Any:
        """public_key is the public key of the signer. It is optional for accounts
        that already exist in state. If unset, the verifier can use the required \\
        signer address for this position and lookup the public key.
        """
    @property
    def mode_info(self) -> global___ModeInfo:
        """mode_info describes the signing mode of the signer and is a nested
        structure to support nested multisig pubkey's
        """
    sequence: builtins.int
    """sequence is the sequence of the account, which describes the
    number of committed transactions signed by a given address. It is used to
    prevent replay attacks.
    """
    def __init__(
        self,
        *,
        public_key: google.protobuf.any_pb2.Any | None = ...,
        mode_info: global___ModeInfo | None = ...,
        sequence: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["mode_info", b"mode_info", "public_key", b"public_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["mode_info", b"mode_info", "public_key", b"public_key", "sequence", b"sequence"]) -> None: ...

global___SignerInfo = SignerInfo

@typing_extensions.final
class ModeInfo(google.protobuf.message.Message):
    """ModeInfo describes the signing mode of a single or nested multisig signer."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class Single(google.protobuf.message.Message):
        """Single is the mode info for a single signer. It is structured as a message
        to allow for additional fields such as locale for SIGN_MODE_TEXTUAL in the
        future
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        MODE_FIELD_NUMBER: builtins.int
        mode: cosmos.tx.signing.v1beta1.signing_pb2.SignMode.ValueType
        """mode is the signing mode of the single signer"""
        def __init__(
            self,
            *,
            mode: cosmos.tx.signing.v1beta1.signing_pb2.SignMode.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["mode", b"mode"]) -> None: ...

    @typing_extensions.final
    class Multi(google.protobuf.message.Message):
        """Multi is the mode info for a multisig public key"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        BITARRAY_FIELD_NUMBER: builtins.int
        MODE_INFOS_FIELD_NUMBER: builtins.int
        @property
        def bitarray(self) -> cosmos.crypto.multisig.v1beta1.multisig_pb2.CompactBitArray:
            """bitarray specifies which keys within the multisig are signing"""
        @property
        def mode_infos(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ModeInfo]:
            """mode_infos is the corresponding modes of the signers of the multisig
            which could include nested multisig public keys
            """
        def __init__(
            self,
            *,
            bitarray: cosmos.crypto.multisig.v1beta1.multisig_pb2.CompactBitArray | None = ...,
            mode_infos: collections.abc.Iterable[global___ModeInfo] | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["bitarray", b"bitarray"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["bitarray", b"bitarray", "mode_infos", b"mode_infos"]) -> None: ...

    SINGLE_FIELD_NUMBER: builtins.int
    MULTI_FIELD_NUMBER: builtins.int
    @property
    def single(self) -> global___ModeInfo.Single:
        """single represents a single signer"""
    @property
    def multi(self) -> global___ModeInfo.Multi:
        """multi represents a nested multisig signer"""
    def __init__(
        self,
        *,
        single: global___ModeInfo.Single | None = ...,
        multi: global___ModeInfo.Multi | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["multi", b"multi", "single", b"single", "sum", b"sum"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["multi", b"multi", "single", b"single", "sum", b"sum"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["sum", b"sum"]) -> typing_extensions.Literal["single", "multi"] | None: ...

global___ModeInfo = ModeInfo

@typing_extensions.final
class Fee(google.protobuf.message.Message):
    """Fee includes the amount of coins paid in fees and the maximum
    gas to be used by the transaction. The ratio yields an effective "gasprice",
    which must be above some miminum to be accepted into the mempool.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AMOUNT_FIELD_NUMBER: builtins.int
    GAS_LIMIT_FIELD_NUMBER: builtins.int
    PAYER_FIELD_NUMBER: builtins.int
    GRANTER_FIELD_NUMBER: builtins.int
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """amount is the amount of coins to be paid as a fee"""
    gas_limit: builtins.int
    """gas_limit is the maximum gas that can be used in transaction processing
    before an out of gas error occurs
    """
    payer: builtins.str
    """if unset, the first signer is responsible for paying the fees. If set, the specified account must pay the fees.
    the payer must be a tx signer (and thus have signed this field in AuthInfo).
    setting this field does *not* change the ordering of required signers for the transaction.
    """
    granter: builtins.str
    """if set, the fee payer (either the first signer or the value of the payer field) requests that a fee grant be used
    to pay fees instead of the fee payer's own balance. If an appropriate fee grant does not exist or the chain does
    not support fee grants, this will fail
    """
    def __init__(
        self,
        *,
        amount: collections.abc.Iterable[cosmos.base.v1beta1.coin_pb2.Coin] | None = ...,
        gas_limit: builtins.int = ...,
        payer: builtins.str = ...,
        granter: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount", b"amount", "gas_limit", b"gas_limit", "granter", b"granter", "payer", b"payer"]) -> None: ...

global___Fee = Fee



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import osmosis.incentives.gauge_pb2
import osmosis.incentives.params_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the incentives module's various parameters when first
    initialized
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARAMS_FIELD_NUMBER: builtins.int
    GAUGES_FIELD_NUMBER: builtins.int
    LOCKABLE_DURATIONS_FIELD_NUMBER: builtins.int
    LAST_GAUGE_ID_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> osmosis.incentives.params_pb2.Params:
        """params are all the parameters of the module"""
    @property
    def gauges(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.incentives.gauge_pb2.Gauge]:
        """gauges are all gauges that should exist at genesis"""
    @property
    def lockable_durations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.duration_pb2.Duration]:
        """lockable_durations are all lockup durations that gauges can be locked for
        in order to recieve incentives
        """
    last_gauge_id: builtins.int
    """last_gauge_id is what the gauge number will increment from when creating
    the next gauge after genesis
    """
    def __init__(
        self,
        *,
        params: osmosis.incentives.params_pb2.Params | None = ...,
        gauges: collections.abc.Iterable[osmosis.incentives.gauge_pb2.Gauge] | None = ...,
        lockable_durations: collections.abc.Iterable[google.protobuf.duration_pb2.Duration] | None = ...,
        last_gauge_id: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params", b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["gauges", b"gauges", "last_gauge_id", b"last_gauge_id", "lockable_durations", b"lockable_durations", "params", b"params"]) -> None: ...

global___GenesisState = GenesisState



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import osmosis.lockup.lock_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the lockup module's genesis state."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LAST_LOCK_ID_FIELD_NUMBER: builtins.int
    LOCKS_FIELD_NUMBER: builtins.int
    SYNTHETIC_LOCKS_FIELD_NUMBER: builtins.int
    last_lock_id: builtins.int
    @property
    def locks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.lockup.lock_pb2.PeriodLock]: ...
    @property
    def synthetic_locks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.lockup.lock_pb2.SyntheticLock]: ...
    def __init__(
        self,
        *,
        last_lock_id: builtins.int = ...,
        locks: collections.abc.Iterable[osmosis.lockup.lock_pb2.PeriodLock] | None = ...,
        synthetic_locks: collections.abc.Iterable[osmosis.lockup.lock_pb2.SyntheticLock] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["last_lock_id", b"last_lock_id", "locks", b"locks", "synthetic_locks", b"synthetic_locks"]) -> None: ...

global___GenesisState = GenesisState



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/gamm/pool-models/balancer/tx/tx.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from osmosis.gamm.pool_models.balancer import balancerPool_pb2 as osmosis_dot_gamm_dot_pool__models_dot_balancer_dot_balancerPool__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-osmosis/gamm/pool-models/balancer/tx/tx.proto\x12(osmosis.gamm.poolmodels.balancer.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x34osmosis/gamm/pool-models/balancer/balancerPool.proto\"\x84\x02\n\x15MsgCreateBalancerPool\x12!\n\x06sender\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"sender\"\x12M\n\x0bpool_params\x18\x02 \x01(\x0b\x32 .osmosis.gamm.v1beta1.PoolParamsB\x16\xf2\xde\x1f\x12yaml:\"pool_params\"\x12:\n\x0bpool_assets\x18\x03 \x03(\x0b\x32\x1f.osmosis.gamm.v1beta1.PoolAssetB\x04\xc8\xde\x1f\x00\x12=\n\x14\x66uture_pool_governor\x18\x04 \x01(\tB\x1f\xf2\xde\x1f\x1byaml:\"future_pool_governor\"\"<\n\x1dMsgCreateBalancerPoolResponse\x12\x1b\n\x07pool_id\x18\x01 \x01(\x04\x42\n\xe2\xde\x1f\x06PoolID2\xa6\x01\n\x03Msg\x12\x9e\x01\n\x12\x43reateBalancerPool\x12?.osmosis.gamm.poolmodels.balancer.v1beta1.MsgCreateBalancerPool\x1aG.osmosis.gamm.poolmodels.balancer.v1beta1.MsgCreateBalancerPoolResponseBAZ?github.com/osmosis-labs/osmosis/v14/x/gamm/pool-models/balancerb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.gamm.pool_models.balancer.tx.tx_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z?github.com/osmosis-labs/osmosis/v14/x/gamm/pool-models/balancer'
  _MSGCREATEBALANCERPOOL.fields_by_name['sender']._options = None
  _MSGCREATEBALANCERPOOL.fields_by_name['sender']._serialized_options = b'\362\336\037\ryaml:\"sender\"'
  _MSGCREATEBALANCERPOOL.fields_by_name['pool_params']._options = None
  _MSGCREATEBALANCERPOOL.fields_by_name['pool_params']._serialized_options = b'\362\336\037\022yaml:\"pool_params\"'
  _MSGCREATEBALANCERPOOL.fields_by_name['pool_assets']._options = None
  _MSGCREATEBALANCERPOOL.fields_by_name['pool_assets']._serialized_options = b'\310\336\037\000'
  _MSGCREATEBALANCERPOOL.fields_by_name['future_pool_governor']._options = None
  _MSGCREATEBALANCERPOOL.fields_by_name['future_pool_governor']._serialized_options = b'\362\336\037\033yaml:\"future_pool_governor\"'
  _MSGCREATEBALANCERPOOLRESPONSE.fields_by_name['pool_id']._options = None
  _MSGCREATEBALANCERPOOLRESPONSE.fields_by_name['pool_id']._serialized_options = b'\342\336\037\006PoolID'
  _MSGCREATEBALANCERPOOL._serialized_start=168
  _MSGCREATEBALANCERPOOL._serialized_end=428
  _MSGCREATEBALANCERPOOLRESPONSE._serialized_start=430
  _MSGCREATEBALANCERPOOLRESPONSE._serialized_end=490
  _MSG._serialized_start=493
  _MSG._serialized_end=659
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.distribution.v1beta1.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service for distribution module."""

    def __init__(self, channel: grpc.Channel) -> None: ...
    Params: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryParamsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryParamsResponse,
    ]
    """Params queries params of the distribution module."""
    ValidatorOutstandingRewards: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsResponse,
    ]
    """ValidatorOutstandingRewards queries rewards of a validator address."""
    ValidatorCommission: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionResponse,
    ]
    """ValidatorCommission queries accumulated commission for a validator."""
    ValidatorSlashes: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesResponse,
    ]
    """ValidatorSlashes queries slash events of a validator."""
    DelegationRewards: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsResponse,
    ]
    """DelegationRewards queries the total rewards accrued by a delegation."""
    DelegationTotalRewards: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsResponse,
    ]
    """DelegationTotalRewards queries the total rewards accrued by a each
    validator.
    """
    DelegatorValidators: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsResponse,
    ]
    """DelegatorValidators queries the validators of a delegator."""
    DelegatorWithdrawAddress: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressResponse,
    ]
    """DelegatorWithdrawAddress queries withdraw address of a delegator."""
    CommunityPool: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolResponse,
    ]
    """CommunityPool queries the community pool coins."""

class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service for distribution module."""

    @abc.abstractmethod
    def Params(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries params of the distribution module."""
    @abc.abstractmethod
    def ValidatorOutstandingRewards(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsResponse:
        """ValidatorOutstandingRewards queries rewards of a validator address."""
    @abc.abstractmethod
    def ValidatorCommission(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionResponse:
        """ValidatorCommission queries accumulated commission for a validator."""
    @abc.abstractmethod
    def ValidatorSlashes(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesResponse:
        """ValidatorSlashes queries slash events of a validator."""
    @abc.abstractmethod
    def DelegationRewards(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsResponse:
        """DelegationRewards queries the total rewards accrued by a delegation."""
    @abc.abstractmethod
    def DelegationTotalRewards(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsResponse:
        """DelegationTotalRewards queries the total rewards accrued by a each
        validator.
        """
    @abc.abstractmethod
    def DelegatorValidators(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsResponse:
        """DelegatorValidators queries the validators of a delegator."""
    @abc.abstractmethod
    def DelegatorWithdrawAddress(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressResponse:
        """DelegatorWithdrawAddress queries withdraw address of a delegator."""
    @abc.abstractmethod
    def CommunityPool(
        self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolResponse:
        """CommunityPool queries the community pool coins."""

def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.mint.v1beta1.mint_pb2
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the mint module's genesis state."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MINTER_FIELD_NUMBER: builtins.int
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def minter(self) -> cosmos.mint.v1beta1.mint_pb2.Minter:
        """minter is a space for holding current inflation information."""
    @property
    def params(self) -> cosmos.mint.v1beta1.mint_pb2.Params:
        """params defines all the paramaters of the module."""
    def __init__(
        self,
        *,
        minter: cosmos.mint.v1beta1.mint_pb2.Minter | None = ...,
        params: cosmos.mint.v1beta1.mint_pb2.Params | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["minter", b"minter", "params", b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["minter", b"minter", "params", b"params"]) -> None: ...

global___GenesisState = GenesisState



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import osmosis.ibc_rate_limit.v1beta1.params_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ParamsRequest(google.protobuf.message.Message):
    """ParamsRequest is the request type for the Query/Params RPC method."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ParamsRequest = ParamsRequest

@typing_extensions.final
class ParamsResponse(google.protobuf.message.Message):
    """aramsResponse is the response type for the Query/Params RPC method."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> osmosis.ibc_rate_limit.v1beta1.params_pb2.Params:
        """params defines the parameters of the module."""
    def __init__(
        self,
        *,
        params: osmosis.ibc_rate_limit.v1beta1.params_pb2.Params | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params", b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params", b"params"]) -> None: ...

global___ParamsResponse = ParamsResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/bank/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google_apis.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos.bank.v1beta1 import bank_pb2 as cosmos_dot_bank_dot_v1beta1_dot_bank__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x63osmos/bank/v1beta1/query.proto\x12\x13\x63osmos.bank.v1beta1\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x1e\x63osmos/bank/v1beta1/bank.proto\"?\n\x13QueryBalanceRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x02 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"B\n\x14QueryBalanceResponse\x12*\n\x07\x62\x61lance\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.Coin\"p\n\x17QueryAllBalancesRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12:\n\npagination\x18\x02 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xb6\x01\n\x18QueryAllBalancesResponse\x12]\n\x08\x62\x61lances\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"_\n\x17QueryTotalSupplyRequest\x12:\n\npagination\x18\x01 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xb4\x01\n\x18QueryTotalSupplyResponse\x12[\n\x06supply\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"%\n\x14QuerySupplyOfRequest\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\"H\n\x15QuerySupplyOfResponse\x12/\n\x06\x61mount\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"l\n$QueryTotalSupplyWithoutOffsetRequest\x12:\n\npagination\x18\x01 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xc1\x01\n%QueryTotalSupplyWithoutOffsetResponse\x12[\n\x06supply\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"2\n!QuerySupplyOfWithoutOffsetRequest\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\"U\n\"QuerySupplyOfWithoutOffsetResponse\x12/\n\x06\x61mount\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x14\n\x12QueryParamsRequest\"H\n\x13QueryParamsResponse\x12\x31\n\x06params\x18\x01 \x01(\x0b\x32\x1b.cosmos.bank.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\"X\n\x1aQueryDenomsMetadataRequest\x12:\n\npagination\x18\x01 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest\"\x92\x01\n\x1bQueryDenomsMetadataResponse\x12\x36\n\tmetadatas\x18\x01 \x03(\x0b\x32\x1d.cosmos.bank.v1beta1.MetadataB\x04\xc8\xde\x1f\x00\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"*\n\x19QueryDenomMetadataRequest\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\"S\n\x1aQueryDenomMetadataResponse\x12\x35\n\x08metadata\x18\x01 \x01(\x0b\x32\x1d.cosmos.bank.v1beta1.MetadataB\x04\xc8\xde\x1f\x00\"&\n\x15QueryBaseDenomRequest\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\",\n\x16QueryBaseDenomResponse\x12\x12\n\nbase_denom\x18\x01 \x01(\t2\xd2\x0c\n\x05Query\x12\x98\x01\n\x07\x42\x61lance\x12(.cosmos.bank.v1beta1.QueryBalanceRequest\x1a).cosmos.bank.v1beta1.QueryBalanceResponse\"8\x82\xd3\xe4\x93\x02\x32\x12\x30/cosmos/bank/v1beta1/balances/{address}/by_denom\x12\x9b\x01\n\x0b\x41llBalances\x12,.cosmos.bank.v1beta1.QueryAllBalancesRequest\x1a-.cosmos.bank.v1beta1.QueryAllBalancesResponse\"/\x82\xd3\xe4\x93\x02)\x12\'/cosmos/bank/v1beta1/balances/{address}\x12\x8f\x01\n\x0bTotalSupply\x12,.cosmos.bank.v1beta1.QueryTotalSupplyRequest\x1a-.cosmos.bank.v1beta1.QueryTotalSupplyResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/cosmos/bank/v1beta1/supply\x12\x8e\x01\n\x08SupplyOf\x12).cosmos.bank.v1beta1.QuerySupplyOfRequest\x1a*.cosmos.bank.v1beta1.QuerySupplyOfResponse\"+\x82\xd3\xe4\x93\x02%\x12#/cosmos/bank/v1beta1/supply/{denom}\x12\xc5\x01\n\x18TotalSupplyWithoutOffset\x12\x39.cosmos.bank.v1beta1.QueryTotalSupplyWithoutOffsetRequest\x1a:.cosmos.bank.v1beta1.QueryTotalSupplyWithoutOffsetResponse\"2\x82\xd3\xe4\x93\x02,\x12*/cosmos/bank/v1beta1/supply_without_offset\x12\xc4\x01\n\x15SupplyOfWithoutOffset\x12\x36.cosmos.bank.v1beta1.QuerySupplyOfWithoutOffsetRequest\x1a\x37.cosmos.bank.v1beta1.QuerySupplyOfWithoutOffsetResponse\":\x82\xd3\xe4\x93\x02\x34\x12\x32/cosmos/bank/v1beta1/supply_without_offset/{denom}\x12\x80\x01\n\x06Params\x12\'.cosmos.bank.v1beta1.QueryParamsRequest\x1a(.cosmos.bank.v1beta1.QueryParamsResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/cosmos/bank/v1beta1/params\x12\xa6\x01\n\rDenomMetadata\x12..cosmos.bank.v1beta1.QueryDenomMetadataRequest\x1a/.cosmos.bank.v1beta1.QueryDenomMetadataResponse\"4\x82\xd3\xe4\x93\x02.\x12,/cosmos/bank/v1beta1/denoms_metadata/{denom}\x12\xa1\x01\n\x0e\x44\x65nomsMetadata\x12/.cosmos.bank.v1beta1.QueryDenomsMetadataRequest\x1a\x30.cosmos.bank.v1beta1.QueryDenomsMetadataResponse\",\x82\xd3\xe4\x93\x02&\x12$/cosmos/bank/v1beta1/denoms_metadata\x12\x8d\x01\n\tBaseDenom\x12*.cosmos.bank.v1beta1.QueryBaseDenomRequest\x1a+.cosmos.bank.v1beta1.QueryBaseDenomResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/cosmos/bank/v1beta1/base_denomB+Z)github.com/cosmos/cosmos-sdk/x/bank/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.bank.v1beta1.query_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z)github.com/cosmos/cosmos-sdk/x/bank/types'
  _QUERYBALANCEREQUEST._options = None
  _QUERYBALANCEREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYALLBALANCESREQUEST._options = None
  _QUERYALLBALANCESREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYALLBALANCESRESPONSE.fields_by_name['balances']._options = None
  _QUERYALLBALANCESRESPONSE.fields_by_name['balances']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _QUERYTOTALSUPPLYREQUEST._options = None
  _QUERYTOTALSUPPLYREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYTOTALSUPPLYRESPONSE.fields_by_name['supply']._options = None
  _QUERYTOTALSUPPLYRESPONSE.fields_by_name['supply']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _QUERYSUPPLYOFRESPONSE.fields_by_name['amount']._options = None
  _QUERYSUPPLYOFRESPONSE.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _QUERYTOTALSUPPLYWITHOUTOFFSETREQUEST._options = None
  _QUERYTOTALSUPPLYWITHOUTOFFSETREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYTOTALSUPPLYWITHOUTOFFSETRESPONSE.fields_by_name['supply']._options = None
  _QUERYTOTALSUPPLYWITHOUTOFFSETRESPONSE.fields_by_name['supply']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _QUERYSUPPLYOFWITHOUTOFFSETRESPONSE.fields_by_name['amount']._options = None
  _QUERYSUPPLYOFWITHOUTOFFSETRESPONSE.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYDENOMSMETADATARESPONSE.fields_by_name['metadatas']._options = None
  _QUERYDENOMSMETADATARESPONSE.fields_by_name['metadatas']._serialized_options = b'\310\336\037\000'
  _QUERYDENOMMETADATARESPONSE.fields_by_name['metadata']._options = None
  _QUERYDENOMMETADATARESPONSE.fields_by_name['metadata']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['Balance']._options = None
  _QUERY.methods_by_name['Balance']._serialized_options = b'\202\323\344\223\0022\0220/cosmos/bank/v1beta1/balances/{address}/by_denom'
  _QUERY.methods_by_name['AllBalances']._options = None
  _QUERY.methods_by_name['AllBalances']._serialized_options = b'\202\323\344\223\002)\022\'/cosmos/bank/v1beta1/balances/{address}'
  _QUERY.methods_by_name['TotalSupply']._options = None
  _QUERY.methods_by_name['TotalSupply']._serialized_options = b'\202\323\344\223\002\035\022\033/cosmos/bank/v1beta1/supply'
  _QUERY.methods_by_name['SupplyOf']._options = None
  _QUERY.methods_by_name['SupplyOf']._serialized_options = b'\202\323\344\223\002%\022#/cosmos/bank/v1beta1/supply/{denom}'
  _QUERY.methods_by_name['TotalSupplyWithoutOffset']._options = None
  _QUERY.methods_by_name['TotalSupplyWithoutOffset']._serialized_options = b'\202\323\344\223\002,\022*/cosmos/bank/v1beta1/supply_without_offset'
  _QUERY.methods_by_name['SupplyOfWithoutOffset']._options = None
  _QUERY.methods_by_name['SupplyOfWithoutOffset']._serialized_options = b'\202\323\344\223\0024\0222/cosmos/bank/v1beta1/supply_without_offset/{denom}'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\035\022\033/cosmos/bank/v1beta1/params'
  _QUERY.methods_by_name['DenomMetadata']._options = None
  _QUERY.methods_by_name['DenomMetadata']._serialized_options = b'\202\323\344\223\002.\022,/cosmos/bank/v1beta1/denoms_metadata/{denom}'
  _QUERY.methods_by_name['DenomsMetadata']._options = None
  _QUERY.methods_by_name['DenomsMetadata']._serialized_options = b'\202\323\344\223\002&\022$/cosmos/bank/v1beta1/denoms_metadata'
  _QUERY.methods_by_name['BaseDenom']._options = None
  _QUERY.methods_by_name['BaseDenom']._serialized_options = b'\202\323\344\223\002!\022\037/cosmos/bank/v1beta1/base_denom'
  _QUERYBALANCEREQUEST._serialized_start=216
  _QUERYBALANCEREQUEST._serialized_end=279
  _QUERYBALANCERESPONSE._serialized_start=281
  _QUERYBALANCERESPONSE._serialized_end=347
  _QUERYALLBALANCESREQUEST._serialized_start=349
  _QUERYALLBALANCESREQUEST._serialized_end=461
  _QUERYALLBALANCESRESPONSE._serialized_start=464
  _QUERYALLBALANCESRESPONSE._serialized_end=646
  _QUERYTOTALSUPPLYREQUEST._serialized_start=648
  _QUERYTOTALSUPPLYREQUEST._serialized_end=743
  _QUERYTOTALSUPPLYRESPONSE._serialized_start=746
  _QUERYTOTALSUPPLYRESPONSE._serialized_end=926
  _QUERYSUPPLYOFREQUEST._serialized_start=928
  _QUERYSUPPLYOFREQUEST._serialized_end=965
  _QUERYSUPPLYOFRESPONSE._serialized_start=967
  _QUERYSUPPLYOFRESPONSE._serialized_end=1039
  _QUERYTOTALSUPPLYWITHOUTOFFSETREQUEST._serialized_start=1041
  _QUERYTOTALSUPPLYWITHOUTOFFSETREQUEST._serialized_end=1149
  _QUERYTOTALSUPPLYWITHOUTOFFSETRESPONSE._serialized_start=1152
  _QUERYTOTALSUPPLYWITHOUTOFFSETRESPONSE._serialized_end=1345
  _QUERYSUPPLYOFWITHOUTOFFSETREQUEST._serialized_start=1347
  _QUERYSUPPLYOFWITHOUTOFFSETREQUEST._serialized_end=1397
  _QUERYSUPPLYOFWITHOUTOFFSETRESPONSE._serialized_start=1399
  _QUERYSUPPLYOFWITHOUTOFFSETRESPONSE._serialized_end=1484
  _QUERYPARAMSREQUEST._serialized_start=1486
  _QUERYPARAMSREQUEST._serialized_end=1506
  _QUERYPARAMSRESPONSE._serialized_start=1508
  _QUERYPARAMSRESPONSE._serialized_end=1580
  _QUERYDENOMSMETADATAREQUEST._serialized_start=1582
  _QUERYDENOMSMETADATAREQUEST._serialized_end=1670
  _QUERYDENOMSMETADATARESPONSE._serialized_start=1673
  _QUERYDENOMSMETADATARESPONSE._serialized_end=1819
  _QUERYDENOMMETADATAREQUEST._serialized_start=1821
  _QUERYDENOMMETADATAREQUEST._serialized_end=1863
  _QUERYDENOMMETADATARESPONSE._serialized_start=1865
  _QUERYDENOMMETADATARESPONSE._serialized_end=1948
  _QUERYBASEDENOMREQUEST._serialized_start=1950
  _QUERYBASEDENOMREQUEST._serialized_end=1988
  _QUERYBASEDENOMRESPONSE._serialized_start=1990
  _QUERYBASEDENOMRESPONSE._serialized_end=2034
  _QUERY._serialized_start=2037
  _QUERY._serialized_end=3655
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/gamm/v1beta1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"osmosis/gamm/v1beta1/genesis.proto\x12\x14osmosis.gamm.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\"\x8d\x01\n\x06Params\x12\x82\x01\n\x11pool_creation_fee\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinBL\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\xf2\xde\x1f\x18yaml:\"pool_creation_fee\"\xc8\xde\x1f\x00\"\x8c\x01\n\x0cGenesisState\x12.\n\x05pools\x18\x01 \x03(\x0b\x32\x14.google.protobuf.AnyB\t\xca\xb4-\x05PoolI\x12\x18\n\x10next_pool_number\x18\x02 \x01(\x04\x12\x32\n\x06params\x18\x03 \x01(\x0b\x32\x1c.osmosis.gamm.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\x42\x32Z0github.com/osmosis-labs/osmosis/v14/x/gamm/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.gamm.v1beta1.genesis_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z0github.com/osmosis-labs/osmosis/v14/x/gamm/types'
  _PARAMS.fields_by_name['pool_creation_fee']._options = None
  _PARAMS.fields_by_name['pool_creation_fee']._serialized_options = b'\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins\362\336\037\030yaml:\"pool_creation_fee\"\310\336\037\000'
  _GENESISSTATE.fields_by_name['pools']._options = None
  _GENESISSTATE.fields_by_name['pools']._serialized_options = b'\312\264-\005PoolI'
  _GENESISSTATE.fields_by_name['params']._options = None
  _GENESISSTATE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _PARAMS._serialized_start=169
  _PARAMS._serialized_end=310
  _GENESISSTATE._serialized_start=313
  _GENESISSTATE._serialized_end=453
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import osmosis.txfees.v1beta1.feetoken_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class UpdateFeeTokenProposal(google.protobuf.message.Message):
    """UpdateFeeTokenProposal is a gov Content type for adding a new whitelisted fee
    token. It must specify a denom along with gamm pool ID to use as a spot price
    calculator. It can be used to add a new denom to the whitelist It can also be
    used to update the Pool to associate with the denom. If Pool ID is set to 0,
    it will remove the denom from the whitelisted set.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    FEETOKEN_FIELD_NUMBER: builtins.int
    title: builtins.str
    description: builtins.str
    @property
    def feetoken(self) -> osmosis.txfees.v1beta1.feetoken_pb2.FeeToken: ...
    def __init__(
        self,
        *,
        title: builtins.str = ...,
        description: builtins.str = ...,
        feetoken: osmosis.txfees.v1beta1.feetoken_pb2.FeeToken | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["feetoken", b"feetoken"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description", "feetoken", b"feetoken", "title", b"title"]) -> None: ...

global___UpdateFeeTokenProposal = UpdateFeeTokenProposal



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/twap/v1beta1/twap_record.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&osmosis/twap/v1beta1/twap_record.proto\x12\x14osmosis.twap.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xc3\x05\n\nTwapRecord\x12\x0f\n\x07pool_id\x18\x01 \x01(\x04\x12\x14\n\x0c\x61sset0_denom\x18\x02 \x01(\t\x12\x14\n\x0c\x61sset1_denom\x18\x03 \x01(\t\x12\x39\n\x06height\x18\x04 \x01(\x03\x42)\xf2\xde\x1f\x14yaml:\"record_height\"\xea\xde\x1f\rrecord_height\x12H\n\x04time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x1e\xc8\xde\x1f\x00\x90\xdf\x1f\x01\xf2\xde\x1f\x12yaml:\"record_time\"\x12J\n\x12p0_last_spot_price\x18\x06 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12J\n\x12p1_last_spot_price\x18\x07 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12V\n\x1ep0_arithmetic_twap_accumulator\x18\x08 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12V\n\x1ep1_arithmetic_twap_accumulator\x18\t \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12R\n\x1ageometric_twap_accumulator\x18\n \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12W\n\x0flast_error_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampB\"\xc8\xde\x1f\x00\x90\xdf\x1f\x01\xf2\xde\x1f\x16yaml:\"last_error_time\"B2Z0github.com/osmosis-labs/osmosis/v14/x/twap/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.twap.v1beta1.twap_record_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z0github.com/osmosis-labs/osmosis/v14/x/twap/types'
  _TWAPRECORD.fields_by_name['height']._options = None
  _TWAPRECORD.fields_by_name['height']._serialized_options = b'\362\336\037\024yaml:\"record_height\"\352\336\037\rrecord_height'
  _TWAPRECORD.fields_by_name['time']._options = None
  _TWAPRECORD.fields_by_name['time']._serialized_options = b'\310\336\037\000\220\337\037\001\362\336\037\022yaml:\"record_time\"'
  _TWAPRECORD.fields_by_name['p0_last_spot_price']._options = None
  _TWAPRECORD.fields_by_name['p0_last_spot_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TWAPRECORD.fields_by_name['p1_last_spot_price']._options = None
  _TWAPRECORD.fields_by_name['p1_last_spot_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TWAPRECORD.fields_by_name['p0_arithmetic_twap_accumulator']._options = None
  _TWAPRECORD.fields_by_name['p0_arithmetic_twap_accumulator']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TWAPRECORD.fields_by_name['p1_arithmetic_twap_accumulator']._options = None
  _TWAPRECORD.fields_by_name['p1_arithmetic_twap_accumulator']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TWAPRECORD.fields_by_name['geometric_twap_accumulator']._options = None
  _TWAPRECORD.fields_by_name['geometric_twap_accumulator']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TWAPRECORD.fields_by_name['last_error_time']._options = None
  _TWAPRECORD.fields_by_name['last_error_time']._serialized_options = b'\310\336\037\000\220\337\037\001\362\336\037\026yaml:\"last_error_time\"'
  _TWAPRECORD._serialized_start=206
  _TWAPRECORD._serialized_end=913
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/downtime-detector/v1beta1/downtime_duration.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9osmosis/downtime-detector/v1beta1/downtime_duration.proto\x12 osmosis.downtimedetector.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto*\xc2\x03\n\x08\x44owntime\x12\x10\n\x0c\x44URATION_30S\x10\x00\x12\x0f\n\x0b\x44URATION_1M\x10\x01\x12\x0f\n\x0b\x44URATION_2M\x10\x02\x12\x0f\n\x0b\x44URATION_3M\x10\x03\x12\x0f\n\x0b\x44URATION_4M\x10\x04\x12\x0f\n\x0b\x44URATION_5M\x10\x05\x12\x10\n\x0c\x44URATION_10M\x10\x06\x12\x10\n\x0c\x44URATION_20M\x10\x07\x12\x10\n\x0c\x44URATION_30M\x10\x08\x12\x10\n\x0c\x44URATION_40M\x10\t\x12\x10\n\x0c\x44URATION_50M\x10\n\x12\x0f\n\x0b\x44URATION_1H\x10\x0b\x12\x11\n\rDURATION_1_5H\x10\x0c\x12\x0f\n\x0b\x44URATION_2H\x10\r\x12\x11\n\rDURATION_2_5H\x10\x0e\x12\x0f\n\x0b\x44URATION_3H\x10\x0f\x12\x0f\n\x0b\x44URATION_4H\x10\x10\x12\x0f\n\x0b\x44URATION_5H\x10\x11\x12\x0f\n\x0b\x44URATION_6H\x10\x12\x12\x0f\n\x0b\x44URATION_9H\x10\x13\x12\x10\n\x0c\x44URATION_12H\x10\x14\x12\x10\n\x0c\x44URATION_18H\x10\x15\x12\x10\n\x0c\x44URATION_24H\x10\x16\x12\x10\n\x0c\x44URATION_36H\x10\x17\x12\x10\n\x0c\x44URATION_48H\x10\x18\x42?Z=github.com/osmosis-labs/osmosis/v14/x/downtime-detector/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.downtime_detector.v1beta1.downtime_duration_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z=github.com/osmosis-labs/osmosis/v14/x/downtime-detector/types'
  _DOWNTIME._serialized_start=237
  _DOWNTIME._serialized_end=687
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import osmosis.gamm.v2.query_pb2

class QueryStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    SpotPrice: grpc.UnaryUnaryMultiCallable[
        osmosis.gamm.v2.query_pb2.QuerySpotPriceRequest,
        osmosis.gamm.v2.query_pb2.QuerySpotPriceResponse,
    ]
    """SpotPrice defines a gRPC query handler that returns the spot price given
    a base denomination and a quote denomination.
    """

class QueryServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def SpotPrice(
        self,
        request: osmosis.gamm.v2.query_pb2.QuerySpotPriceRequest,
        context: grpc.ServicerContext,
    ) -> osmosis.gamm.v2.query_pb2.QuerySpotPriceResponse:
        """SpotPrice defines a gRPC query handler that returns the spot price given
        a base denomination and a quote denomination.
        """

def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/txfees/v1beta1/feetoken.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%osmosis/txfees/v1beta1/feetoken.proto\x12\x16osmosis.txfees.v1beta1\x1a\x14gogoproto/gogo.proto\"U\n\x08\x46\x65\x65Token\x12\x1f\n\x05\x64\x65nom\x18\x01 \x01(\tB\x10\xf2\xde\x1f\x0cyaml:\"denom\"\x12\"\n\x06poolID\x18\x02 \x01(\x04\x42\x12\xf2\xde\x1f\x0eyaml:\"pool_id\":\x04\xe8\xa0\x1f\x01\x42\x34Z2github.com/osmosis-labs/osmosis/v14/x/txfees/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.txfees.v1beta1.feetoken_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2github.com/osmosis-labs/osmosis/v14/x/txfees/types'
  _FEETOKEN.fields_by_name['denom']._options = None
  _FEETOKEN.fields_by_name['denom']._serialized_options = b'\362\336\037\014yaml:\"denom\"'
  _FEETOKEN.fields_by_name['poolID']._options = None
  _FEETOKEN.fields_by_name['poolID']._serialized_options = b'\362\336\037\016yaml:\"pool_id\"'
  _FEETOKEN._options = None
  _FEETOKEN._serialized_options = b'\350\240\037\001'
  _FEETOKEN._serialized_start=87
  _FEETOKEN._serialized_end=172
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class BaseAccount(google.protobuf.message.Message):
    """BaseAccount defines a base account type. It contains all the necessary fields
    for basic account functionality. Any custom account type should extend this
    type for additional functionality (e.g. vesting).
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    PUB_KEY_FIELD_NUMBER: builtins.int
    ACCOUNT_NUMBER_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    address: builtins.str
    @property
    def pub_key(self) -> google.protobuf.any_pb2.Any: ...
    account_number: builtins.int
    sequence: builtins.int
    def __init__(
        self,
        *,
        address: builtins.str = ...,
        pub_key: google.protobuf.any_pb2.Any | None = ...,
        account_number: builtins.int = ...,
        sequence: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pub_key", b"pub_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_number", b"account_number", "address", b"address", "pub_key", b"pub_key", "sequence", b"sequence"]) -> None: ...

global___BaseAccount = BaseAccount

@typing_extensions.final
class ModuleAccount(google.protobuf.message.Message):
    """ModuleAccount defines an account for modules that holds coins on a pool."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BASE_ACCOUNT_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    PERMISSIONS_FIELD_NUMBER: builtins.int
    @property
    def base_account(self) -> global___BaseAccount: ...
    name: builtins.str
    @property
    def permissions(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        base_account: global___BaseAccount | None = ...,
        name: builtins.str = ...,
        permissions: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_account", b"base_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_account", b"base_account", "name", b"name", "permissions", b"permissions"]) -> None: ...

global___ModuleAccount = ModuleAccount

@typing_extensions.final
class Params(google.protobuf.message.Message):
    """Params defines the parameters for the auth module."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAX_MEMO_CHARACTERS_FIELD_NUMBER: builtins.int
    TX_SIG_LIMIT_FIELD_NUMBER: builtins.int
    TX_SIZE_COST_PER_BYTE_FIELD_NUMBER: builtins.int
    SIG_VERIFY_COST_ED25519_FIELD_NUMBER: builtins.int
    SIG_VERIFY_COST_SECP256K1_FIELD_NUMBER: builtins.int
    max_memo_characters: builtins.int
    tx_sig_limit: builtins.int
    tx_size_cost_per_byte: builtins.int
    sig_verify_cost_ed25519: builtins.int
    sig_verify_cost_secp256k1: builtins.int
    def __init__(
        self,
        *,
        max_memo_characters: builtins.int = ...,
        tx_sig_limit: builtins.int = ...,
        tx_size_cost_per_byte: builtins.int = ...,
        sig_verify_cost_ed25519: builtins.int = ...,
        sig_verify_cost_secp256k1: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_memo_characters", b"max_memo_characters", "sig_verify_cost_ed25519", b"sig_verify_cost_ed25519", "sig_verify_cost_secp256k1", b"sig_verify_cost_secp256k1", "tx_sig_limit", b"tx_sig_limit", "tx_size_cost_per_byte", b"tx_size_cost_per_byte"]) -> None: ...

global___Params = Params



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/sumtree/v1beta1/tree.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"osmosis/sumtree/v1beta1/tree.proto\x12\x15osmosis.store.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\"6\n\x04Node\x12.\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x1c.osmosis.store.v1beta1.Child\"\\\n\x05\x43hild\x12\r\n\x05index\x18\x01 \x01(\x0c\x12\x44\n\x0c\x61\x63\x63umulation\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\xc8\xde\x1f\x00\"2\n\x04Leaf\x12*\n\x04leaf\x18\x01 \x01(\x0b\x32\x1c.osmosis.store.v1beta1.ChildB3Z1github.com/osmosis-labs/osmosis/osmoutils/sumtreeb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.sumtree.v1beta1.tree_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z1github.com/osmosis-labs/osmosis/osmoutils/sumtree'
  _CHILD.fields_by_name['accumulation']._options = None
  _CHILD.fields_by_name['accumulation']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Int\310\336\037\000'
  _NODE._serialized_start=115
  _NODE._serialized_end=169
  _CHILD._serialized_start=171
  _CHILD._serialized_end=263
  _LEAF._serialized_start=265
  _LEAF._serialized_end=315
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Downtime:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DowntimeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Downtime.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DURATION_30S: _Downtime.ValueType  # 0
    DURATION_1M: _Downtime.ValueType  # 1
    DURATION_2M: _Downtime.ValueType  # 2
    DURATION_3M: _Downtime.ValueType  # 3
    DURATION_4M: _Downtime.ValueType  # 4
    DURATION_5M: _Downtime.ValueType  # 5
    DURATION_10M: _Downtime.ValueType  # 6
    DURATION_20M: _Downtime.ValueType  # 7
    DURATION_30M: _Downtime.ValueType  # 8
    DURATION_40M: _Downtime.ValueType  # 9
    DURATION_50M: _Downtime.ValueType  # 10
    DURATION_1H: _Downtime.ValueType  # 11
    DURATION_1_5H: _Downtime.ValueType  # 12
    DURATION_2H: _Downtime.ValueType  # 13
    DURATION_2_5H: _Downtime.ValueType  # 14
    DURATION_3H: _Downtime.ValueType  # 15
    DURATION_4H: _Downtime.ValueType  # 16
    DURATION_5H: _Downtime.ValueType  # 17
    DURATION_6H: _Downtime.ValueType  # 18
    DURATION_9H: _Downtime.ValueType  # 19
    DURATION_12H: _Downtime.ValueType  # 20
    DURATION_18H: _Downtime.ValueType  # 21
    DURATION_24H: _Downtime.ValueType  # 22
    DURATION_36H: _Downtime.ValueType  # 23
    DURATION_48H: _Downtime.ValueType  # 24

class Downtime(_Downtime, metaclass=_DowntimeEnumTypeWrapper): ...

DURATION_30S: Downtime.ValueType  # 0
DURATION_1M: Downtime.ValueType  # 1
DURATION_2M: Downtime.ValueType  # 2
DURATION_3M: Downtime.ValueType  # 3
DURATION_4M: Downtime.ValueType  # 4
DURATION_5M: Downtime.ValueType  # 5
DURATION_10M: Downtime.ValueType  # 6
DURATION_20M: Downtime.ValueType  # 7
DURATION_30M: Downtime.ValueType  # 8
DURATION_40M: Downtime.ValueType  # 9
DURATION_50M: Downtime.ValueType  # 10
DURATION_1H: Downtime.ValueType  # 11
DURATION_1_5H: Downtime.ValueType  # 12
DURATION_2H: Downtime.ValueType  # 13
DURATION_2_5H: Downtime.ValueType  # 14
DURATION_3H: Downtime.ValueType  # 15
DURATION_4H: Downtime.ValueType  # 16
DURATION_5H: Downtime.ValueType  # 17
DURATION_6H: Downtime.ValueType  # 18
DURATION_9H: Downtime.ValueType  # 19
DURATION_12H: Downtime.ValueType  # 20
DURATION_18H: Downtime.ValueType  # 21
DURATION_24H: Downtime.ValueType  # 22
DURATION_36H: Downtime.ValueType  # 23
DURATION_48H: Downtime.ValueType  # 24
global___Downtime = Downtime



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ValidatorSigningInfo(google.protobuf.message.Message):
    """ValidatorSigningInfo defines a validator's signing info for monitoring their
    liveness activity.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    START_HEIGHT_FIELD_NUMBER: builtins.int
    INDEX_OFFSET_FIELD_NUMBER: builtins.int
    JAILED_UNTIL_FIELD_NUMBER: builtins.int
    TOMBSTONED_FIELD_NUMBER: builtins.int
    MISSED_BLOCKS_COUNTER_FIELD_NUMBER: builtins.int
    address: builtins.str
    start_height: builtins.int
    """Height at which validator was first a candidate OR was unjailed"""
    index_offset: builtins.int
    """Index which is incremented each time the validator was a bonded
    in a block and may have signed a precommit or not. This in conjunction with the
    `SignedBlocksWindow` param determines the index in the `MissedBlocksBitArray`.
    """
    @property
    def jailed_until(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Timestamp until which the validator is jailed due to liveness downtime."""
    tombstoned: builtins.bool
    """Whether or not a validator has been tombstoned (killed out of validator set). It is set
    once the validator commits an equivocation or for any other configured misbehiavor.
    """
    missed_blocks_counter: builtins.int
    """A counter kept to avoid unnecessary array reads.
    Note that `Sum(MissedBlocksBitArray)` always equals `MissedBlocksCounter`.
    """
    def __init__(
        self,
        *,
        address: builtins.str = ...,
        start_height: builtins.int = ...,
        index_offset: builtins.int = ...,
        jailed_until: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        tombstoned: builtins.bool = ...,
        missed_blocks_counter: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["jailed_until", b"jailed_until"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "index_offset", b"index_offset", "jailed_until", b"jailed_until", "missed_blocks_counter", b"missed_blocks_counter", "start_height", b"start_height", "tombstoned", b"tombstoned"]) -> None: ...

global___ValidatorSigningInfo = ValidatorSigningInfo

@typing_extensions.final
class Params(google.protobuf.message.Message):
    """Params represents the parameters used for by the slashing module."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SIGNED_BLOCKS_WINDOW_FIELD_NUMBER: builtins.int
    MIN_SIGNED_PER_WINDOW_FIELD_NUMBER: builtins.int
    DOWNTIME_JAIL_DURATION_FIELD_NUMBER: builtins.int
    SLASH_FRACTION_DOUBLE_SIGN_FIELD_NUMBER: builtins.int
    SLASH_FRACTION_DOWNTIME_FIELD_NUMBER: builtins.int
    signed_blocks_window: builtins.int
    min_signed_per_window: builtins.bytes
    @property
    def downtime_jail_duration(self) -> google.protobuf.duration_pb2.Duration: ...
    slash_fraction_double_sign: builtins.bytes
    slash_fraction_downtime: builtins.bytes
    def __init__(
        self,
        *,
        signed_blocks_window: builtins.int = ...,
        min_signed_per_window: builtins.bytes = ...,
        downtime_jail_duration: google.protobuf.duration_pb2.Duration | None = ...,
        slash_fraction_double_sign: builtins.bytes = ...,
        slash_fraction_downtime: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["downtime_jail_duration", b"downtime_jail_duration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["downtime_jail_duration", b"downtime_jail_duration", "min_signed_per_window", b"min_signed_per_window", "signed_blocks_window", b"signed_blocks_window", "slash_fraction_double_sign", b"slash_fraction_double_sign", "slash_fraction_downtime", b"slash_fraction_downtime"]) -> None: ...

global___Params = Params



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmosis/superfluid/tx.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from osmosis.superfluid import superfluid_pb2 as osmosis_dot_superfluid_dot_superfluid__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bosmosis/superfluid/tx.proto\x12\x12osmosis.superfluid\x1a\x14gogoproto/gogo.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a#osmosis/superfluid/superfluid.proto\"]\n\x15MsgSuperfluidDelegate\x12!\n\x06sender\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"sender\"\x12\x0f\n\x07lock_id\x18\x02 \x01(\x04\x12\x10\n\x08val_addr\x18\x03 \x01(\t\"\x1f\n\x1dMsgSuperfluidDelegateResponse\"M\n\x17MsgSuperfluidUndelegate\x12!\n\x06sender\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"sender\"\x12\x0f\n\x07lock_id\x18\x02 \x01(\x04\"!\n\x1fMsgSuperfluidUndelegateResponse\"M\n\x17MsgSuperfluidUnbondLock\x12!\n\x06sender\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"sender\"\x12\x0f\n\x07lock_id\x18\x02 \x01(\x04\"!\n\x1fMsgSuperfluidUnbondLockResponse\"\xaf\x01\n\x1cMsgLockAndSuperfluidDelegate\x12!\n\x06sender\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"sender\"\x12Z\n\x05\x63oins\x18\x02 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\x12\x10\n\x08val_addr\x18\x03 \x01(\t\"2\n$MsgLockAndSuperfluidDelegateResponse\x12\n\n\x02ID\x18\x01 \x01(\x04\"b\n\x18MsgUnPoolWhitelistedPool\x12!\n\x06sender\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"sender\"\x12#\n\x07pool_id\x18\x02 \x01(\x04\x42\x12\xf2\xde\x1f\x0eyaml:\"pool_id\"\";\n MsgUnPoolWhitelistedPoolResponse\x12\x17\n\x0f\x65xited_lock_ids\x18\x01 \x03(\x04\x32\xf4\x04\n\x03Msg\x12r\n\x12SuperfluidDelegate\x12).osmosis.superfluid.MsgSuperfluidDelegate\x1a\x31.osmosis.superfluid.MsgSuperfluidDelegateResponse\x12x\n\x14SuperfluidUndelegate\x12+.osmosis.superfluid.MsgSuperfluidUndelegate\x1a\x33.osmosis.superfluid.MsgSuperfluidUndelegateResponse\x12x\n\x14SuperfluidUnbondLock\x12+.osmosis.superfluid.MsgSuperfluidUnbondLock\x1a\x33.osmosis.superfluid.MsgSuperfluidUnbondLockResponse\x12\x87\x01\n\x19LockAndSuperfluidDelegate\x12\x30.osmosis.superfluid.MsgLockAndSuperfluidDelegate\x1a\x38.osmosis.superfluid.MsgLockAndSuperfluidDelegateResponse\x12{\n\x15UnPoolWhitelistedPool\x12,.osmosis.superfluid.MsgUnPoolWhitelistedPool\x1a\x34.osmosis.superfluid.MsgUnPoolWhitelistedPoolResponseB8Z6github.com/osmosis-labs/osmosis/v14/x/superfluid/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'osmosis.superfluid.tx_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z6github.com/osmosis-labs/osmosis/v14/x/superfluid/types'
  _MSGSUPERFLUIDDELEGATE.fields_by_name['sender']._options = None
  _MSGSUPERFLUIDDELEGATE.fields_by_name['sender']._serialized_options = b'\362\336\037\ryaml:\"sender\"'
  _MSGSUPERFLUIDUNDELEGATE.fields_by_name['sender']._options = None
  _MSGSUPERFLUIDUNDELEGATE.fields_by_name['sender']._serialized_options = b'\362\336\037\ryaml:\"sender\"'
  _MSGSUPERFLUIDUNBONDLOCK.fields_by_name['sender']._options = None
  _MSGSUPERFLUIDUNBONDLOCK.fields_by_name['sender']._serialized_options = b'\362\336\037\ryaml:\"sender\"'
  _MSGLOCKANDSUPERFLUIDDELEGATE.fields_by_name['sender']._options = None
  _MSGLOCKANDSUPERFLUIDDELEGATE.fields_by_name['sender']._serialized_options = b'\362\336\037\ryaml:\"sender\"'
  _MSGLOCKANDSUPERFLUIDDELEGATE.fields_by_name['coins']._options = None
  _MSGLOCKANDSUPERFLUIDDELEGATE.fields_by_name['coins']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _MSGUNPOOLWHITELISTEDPOOL.fields_by_name['sender']._options = None
  _MSGUNPOOLWHITELISTEDPOOL.fields_by_name['sender']._serialized_options = b'\362\336\037\ryaml:\"sender\"'
  _MSGUNPOOLWHITELISTEDPOOL.fields_by_name['pool_id']._options = None
  _MSGUNPOOLWHITELISTEDPOOL.fields_by_name['pool_id']._serialized_options = b'\362\336\037\016yaml:\"pool_id\"'
  _MSGSUPERFLUIDDELEGATE._serialized_start=174
  _MSGSUPERFLUIDDELEGATE._serialized_end=267
  _MSGSUPERFLUIDDELEGATERESPONSE._serialized_start=269
  _MSGSUPERFLUIDDELEGATERESPONSE._serialized_end=300
  _MSGSUPERFLUIDUNDELEGATE._serialized_start=302
  _MSGSUPERFLUIDUNDELEGATE._serialized_end=379
  _MSGSUPERFLUIDUNDELEGATERESPONSE._serialized_start=381
  _MSGSUPERFLUIDUNDELEGATERESPONSE._serialized_end=414
  _MSGSUPERFLUIDUNBONDLOCK._serialized_start=416
  _MSGSUPERFLUIDUNBONDLOCK._serialized_end=493
  _MSGSUPERFLUIDUNBONDLOCKRESPONSE._serialized_start=495
  _MSGSUPERFLUIDUNBONDLOCKRESPONSE._serialized_end=528
  _MSGLOCKANDSUPERFLUIDDELEGATE._serialized_start=531
  _MSGLOCKANDSUPERFLUIDDELEGATE._serialized_end=706
  _MSGLOCKANDSUPERFLUIDDELEGATERESPONSE._serialized_start=708
  _MSGLOCKANDSUPERFLUIDDELEGATERESPONSE._serialized_end=758
  _MSGUNPOOLWHITELISTEDPOOL._serialized_start=760
  _MSGUNPOOLWHITELISTEDPOOL._serialized_end=858
  _MSGUNPOOLWHITELISTEDPOOLRESPONSE._serialized_start=860
  _MSGUNPOOLWHITELISTEDPOOLRESPONSE._serialized_end=919
  _MSG._serialized_start=922
  _MSG._serialized_end=1550
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import osmosis.gamm.pool_models.balancer.balancerPool_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class MsgCreateBalancerPool(google.protobuf.message.Message):
    """===================== MsgCreatePool"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDER_FIELD_NUMBER: builtins.int
    POOL_PARAMS_FIELD_NUMBER: builtins.int
    POOL_ASSETS_FIELD_NUMBER: builtins.int
    FUTURE_POOL_GOVERNOR_FIELD_NUMBER: builtins.int
    sender: builtins.str
    @property
    def pool_params(self) -> osmosis.gamm.pool_models.balancer.balancerPool_pb2.PoolParams: ...
    @property
    def pool_assets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[osmosis.gamm.pool_models.balancer.balancerPool_pb2.PoolAsset]: ...
    future_pool_governor: builtins.str
    def __init__(
        self,
        *,
        sender: builtins.str = ...,
        pool_params: osmosis.gamm.pool_models.balancer.balancerPool_pb2.PoolParams | None = ...,
        pool_assets: collections.abc.Iterable[osmosis.gamm.pool_models.balancer.balancerPool_pb2.PoolAsset] | None = ...,
        future_pool_governor: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_params", b"pool_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["future_pool_governor", b"future_pool_governor", "pool_assets", b"pool_assets", "pool_params", b"pool_params", "sender", b"sender"]) -> None: ...

global___MsgCreateBalancerPool = MsgCreateBalancerPool

@typing_extensions.final
class MsgCreateBalancerPoolResponse(google.protobuf.message.Message):
    """Returns the poolID"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(
        self,
        *,
        pool_id: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id", b"pool_id"]) -> None: ...

global___MsgCreateBalancerPoolResponse = MsgCreateBalancerPoolResponse
